import triad_openvr
import time
import sys
import ctypes
from ctypes import wintypes

# Define the INPUT structure
class MOUSEINPUT(ctypes.Structure):
    _fields_ = [("dx", wintypes.LONG),
                ("dy", wintypes.LONG),
                ("mouseData", wintypes.DWORD),
                ("dwFlags", wintypes.DWORD),
                ("time", wintypes.DWORD),
                ("dwExtraInfo", ctypes.POINTER(wintypes.ULONG))]

class KEYBDINPUT(ctypes.Structure):
    _fields_ = [("wVk", wintypes.WORD),
                ("wScan", wintypes.WORD),
                ("dwFlags", wintypes.DWORD),
                ("time", wintypes.DWORD),
                ("dwExtraInfo", ctypes.POINTER(wintypes.ULONG))]

class HARDWAREINPUT(ctypes.Structure):
    _fields_ = [("uMsg", wintypes.DWORD),
                ("wParamL", wintypes.WORD),
                ("wParamH", wintypes.WORD)]

class INPUT(ctypes.Structure):
    class _INPUT(ctypes.Union):
        _fields_ = [("mi", MOUSEINPUT),
                    ("ki", KEYBDINPUT),
                    ("hi", HARDWAREINPUT)]

    _anonymous_ = ("_input",)
    _fields_ = [("type", wintypes.DWORD),
                ("_input", _INPUT)]

# Constants
INPUT_MOUSE = 0
MOUSEEVENTF_LEFTDOWN = 0x0002
MOUSEEVENTF_LEFTUP = 0x0004

# Create the input events
left_down = INPUT(type=INPUT_MOUSE)
left_down.mi.dwFlags = MOUSEEVENTF_LEFTDOWN

left_up = INPUT(type=INPUT_MOUSE)
left_up.mi.dwFlags = MOUSEEVENTF_LEFTUP

# Get the SendInput function
user32 = ctypes.WinDLL("user32", use_last_error=True)
send_input = user32.SendInput
send_input.restype = wintypes.UINT
send_input.argtypes = (wintypes.UINT, ctypes.POINTER(INPUT), ctypes.c_int)

# Create an array of INPUT structures
input_array = (INPUT * 2)()
input_array[0] = left_down
input_array[1] = left_up

v = triad_openvr.triad_openvr()
v.print_discovered_objects()

if len(sys.argv) == 1:
    interval = 1/250
elif len(sys.argv) == 2:
    interval = 1/float(sys.argv[1])
else:
    print("Invalid number of arguments")
    interval = False

waitRelease = False
    
if interval:
    while(True):
        start = time.time()
        txt = ""
        #print(v.devices.keys())
        if "controller_1" in v.devices.keys():
            get_pose_euler = v.devices["controller_1"].get_pose_euler()
            if False: #get_pose_euler is None:
                print("\r NONE " + str(time.time()), end="")
            else:
                #for each in get_pose_euler:
                #    txt += "%.4f" % each
                #    txt += " "
                #print("\r" + txt, end="")

                ctrlRet = v.devices["controller_1"].get_controller_inputs()

                #for each in ctrlRet.values():
                #    txt += str(each)
                #    txt += " "
                #txt += str(int(ctrlRet["trigger"]))
                #print("\r" + txt, end="")

                if int(ctrlRet["trigger"]) == 1:
                    waitRelease = True
                    # Send the left mouse click (down and up)
                    #send_input(2, input_array, ctypes.sizeof(INPUT))
                    send_input(1, left_down, ctypes.sizeof(INPUT))
                else:
                    if waitRelease == True:
                        waitRelease = False
                        #send_input(2, input_array, ctypes.sizeof(INPUT))
                        send_input(1, left_up, ctypes.sizeof(INPUT))
                                        
                sleep_time = interval-(time.time()-start)
                if sleep_time>0:
                    time.sleep(sleep_time)
